@@ -18,6 +18,7 @@
 import csv
 
 import datasets
+from datasets.tasks import TextClassification
 
 
 _CITATION = """\
@@ -59,6 +60,7 @@ class EmotoneAr(datasets.GeneratorBasedBuilder):
             ),
             homepage=_HOMEPAGE,
             citation=_CITATION,
+            task_templates=[TextClassification(text_column="tweet", label_column="label")],
         )
 
     def _split_generators(self, dl_manager):
