@@ -73,7 +73,7 @@ class EvidenceInferTreatment(datasets.GeneratorBasedBuilder):
             name="2.0",
             description="EvidenceInference V2",
             version=datasets.Version("2.0.0"),
-            zip_file="http://evidence-inference.ebm-nlp.com/v2.0.tar.gz",
+            zip_file="https://github.com/jayded/evidence-inference/archive/refs/tags/v2.0.zip",
         ),
         EvidenceInferenceConfig(
             name="1.1",
@@ -135,34 +135,33 @@ class EvidenceInferTreatment(datasets.GeneratorBasedBuilder):
 
     def _split_generators(self, dl_manager):
         dl_dir = dl_manager.download_and_extract(self.config.zip_file)
-        if self.config.name == "1.1":
-            dl_dir = os.path.join(dl_dir, "evidence-inference-1.1", "annotations")
+        dl_dir = os.path.join(dl_dir, f"evidence-inference-{self.config.name}", "annotations")
 
         SPLITS = {}
         for split in ["train", "test", "validation"]:
             filename = os.path.join(dl_dir, "splits", f"{split}_article_ids.txt")
-            with open(filename, "r", encoding="utf-8") as f:
+            with open(filename, encoding="utf-8") as f:
                 for line in f:
                     id_ = int(line.strip())
                     SPLITS[id_] = split
 
         ALL_PROMPTS = {}
         prompts_filename = os.path.join(dl_dir, "prompts_merged.csv")
-        with open(prompts_filename, "r", encoding="utf-8") as f:
+        with open(prompts_filename, encoding="utf-8") as f:
             data = csv.DictReader(f)
             for item in data:
                 prompt_id = int(item["PromptID"])
                 ALL_PROMPTS[prompt_id] = {"Prompt": item, "Annotations": []}
 
         annotations_filename = os.path.join(dl_dir, "annotations_merged.csv")
-        with open(annotations_filename, "r", encoding="utf-8") as f:
+        with open(annotations_filename, encoding="utf-8") as f:
             data = csv.DictReader(f)
             for item in data:
                 prompt_id = int(item["PromptID"])
-
+                if "" in item:  # Remove unnamed column with row index value
+                    del item[""]
                 if "Annotations" not in ALL_PROMPTS[prompt_id]:
                     ALL_PROMPTS[prompt_id]["Annotations"] = []
-
                 ALL_PROMPTS[prompt_id]["Annotations"].append(item)
 
         # Simplify everything
