@@ -153,14 +153,12 @@ class MsrSQA(datasets.GeneratorBasedBuilder):
 
     def _generate_examples(self, filepath, data_dir):
         """Yields examples."""
-
         with open(filepath, encoding="utf-8") as f:
             reader = csv.DictReader(f, delimiter="\t")
-            for row in reader:
-                item = dict(row)
+            for idx, item in enumerate(reader):
                 item["answer_text"] = _parse_answer_text(item["answer_text"])
                 item["answer_coordinates"] = _parse_answer_coordinates(item["answer_coordinates"])
                 header, table_data = _load_table_data(os.path.join(data_dir, item["table_file"]))
                 item["table_header"] = header
                 item["table_data"] = table_data
-                yield item["id"], item
+                yield idx, item
