@@ -16,12 +16,13 @@
 # Lint as: python3
 """Multi-News dataset."""
 
-
-import os
-
 import datasets
 
 
+_HOMEPAGE = "https://github.com/Alex-Fabbri/Multi-News"
+
+_LICENSE = "For non-commercial research and educational purposes only"
+
 _CITATION = """
 @misc{alex2019multinews,
     title={Multi-News: a Large-Scale Multi-Document Summarization Dataset and Abstractive Hierarchical Model},
@@ -44,7 +45,21 @@ There are two features:
   - summary: news summary.
 """
 
-_URL = "https://drive.google.com/uc?export=download&id=1vRY2wM6rlOZrf9exGTm5pXj5ExlVwJ0C"
+_REPO = "https://huggingface.co/datasets/multi_news/resolve/main/data"
+_URLs = {
+    "train": [
+        f"{_REPO}/train.src.cleaned",
+        f"{_REPO}/train.tgt",
+    ],
+    "val": [
+        f"{_REPO}/val.src.cleaned",
+        f"{_REPO}/val.tgt",
+    ],
+    "test": [
+        f"{_REPO}/test.src.cleaned",
+        f"{_REPO}/test.tgt",
+    ],
+}
 
 _DOCUMENT = "document"
 _SUMMARY = "summary"
@@ -60,39 +75,37 @@ class MultiNews(datasets.GeneratorBasedBuilder):
             description=_DESCRIPTION,
             features=datasets.Features({_DOCUMENT: datasets.Value("string"), _SUMMARY: datasets.Value("string")}),
             supervised_keys=(_DOCUMENT, _SUMMARY),
-            homepage="https://github.com/Alex-Fabbri/Multi-News",
+            homepage=_HOMEPAGE,
+            license=_LICENSE,
             citation=_CITATION,
         )
 
     def _split_generators(self, dl_manager):
         """Returns SplitGenerators."""
-        extract_path = os.path.join(dl_manager.download_and_extract(_URL), "multi-news-original")
+        files = dl_manager.download(_URLs)
         return [
             datasets.SplitGenerator(
                 name=datasets.Split.TRAIN,
-                gen_kwargs={"path": os.path.join(extract_path, "train")},
+                gen_kwargs={"src_file": files["train"][0], "tgt_file": files["train"][1]},
             ),
             datasets.SplitGenerator(
                 name=datasets.Split.VALIDATION,
-                gen_kwargs={"path": os.path.join(extract_path, "val")},
+                gen_kwargs={"src_file": files["val"][0], "tgt_file": files["val"][1]},
             ),
             datasets.SplitGenerator(
                 name=datasets.Split.TEST,
-                gen_kwargs={"path": os.path.join(extract_path, "test")},
+                gen_kwargs={"src_file": files["test"][0], "tgt_file": files["test"][1]},
             ),
         ]
 
-    def _generate_examples(self, path=None):
+    def _generate_examples(self, src_file, tgt_file):
         """Yields examples."""
-        with open(os.path.join(path + ".src"), encoding="utf-8") as src_f, open(
-            os.path.join(path + ".tgt"), encoding="utf-8"
-        ) as tgt_f:
+        with open(src_file, encoding="utf-8") as src_f, open(tgt_file, encoding="utf-8") as tgt_f:
             for i, (src_line, tgt_line) in enumerate(zip(src_f, tgt_f)):
                 yield i, {
                     # In original file, each line has one example and natural newline
                     # tokens "\n" are being replaced with "NEWLINE_CHAR". Here restore
                     # the natural newline token to avoid special vocab "NEWLINE_CHAR".
                     _DOCUMENT: src_line.strip().replace("NEWLINE_CHAR", "\n"),
-                    # Remove the starting token "- " for every target sequence.
-                    _SUMMARY: tgt_line.strip().lstrip("- "),
+                    _SUMMARY: tgt_line.strip(),
                 }
