@@ -77,11 +77,7 @@ class TeluguBooks(datasets.GeneratorBasedBuilder):
         path_to_manual_file = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))
         if not os.path.exists(path_to_manual_file):
             raise FileNotFoundError(
-                "{} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('telugu_books', data_dir=...)` that includes file name {}. Manual download instructions: {}".format(
-                    path_to_manual_file,
-                    _FILENAME,
-                    self.manual_download_instructions,
-                )
+                f"{path_to_manual_file} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('telugu_books', data_dir=...)` that includes file name {_FILENAME}. Manual download instructions: {self.manual_download_instructions}"
             )
         return [
             datasets.SplitGenerator(
