@@ -67,7 +67,7 @@ class XquadR(datasets.GeneratorBasedBuilder):
 
     # TODO(xquad-r): Set up version.
     VERSION = datasets.Version("1.1.0")
-    BUILDER_CONFIGS = [XquadRConfig(name="{}".format(lang), description=_DESCRIPTION, lang=lang) for lang in _LANG]
+    BUILDER_CONFIGS = [XquadRConfig(name=f"{lang}", description=_DESCRIPTION, lang=lang) for lang in _LANG]
 
     def _info(self):
         # TODO(xquad-r): Specifies the datasets.DatasetInfo object
@@ -102,7 +102,7 @@ class XquadR(datasets.GeneratorBasedBuilder):
         # TODO(xquad-r): Downloads the data and defines the splits
         # dl_manager is a datasets.download.DownloadManager that can be used to
         # download and extract URLs
-        urls_to_download = {lang: _URL + "{}.json".format(lang) for lang in _LANG}
+        urls_to_download = {lang: _URL + f"{lang}.json" for lang in _LANG}
         downloaded_files = dl_manager.download_and_extract(urls_to_download)
 
         return [
