@@ -1,5 +1,4 @@
 # flake8: noqa
-# coding=utf-8
 # Copyright 2020 The HuggingFace Datasets Authors and the TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,42 +17,34 @@
 # pylint: enable=line-too-long
 # pylint: disable=g-import-not-at-top,g-bad-import-order,wrong-import-position
 
-__version__ = "1.18.1"
+__version__ = "2.4.0"
 
 import pyarrow
-from packaging import version as _version
-from pyarrow import total_allocated_bytes
+from packaging import version
 
 
-if _version.parse(pyarrow.__version__).major < 3:
+if version.parse(pyarrow.__version__).major < 6:
     raise ImportWarning(
-        "To use `datasets`, the module `pyarrow>=3.0.0` is required, and the current version of `pyarrow` doesn't match this condition.\n"
+        "To use `datasets`, the module `pyarrow>=6.0.0` is required, and the current version of `pyarrow` doesn't match this condition.\n"
         "If you are running this in a Google Colab, you should probably just restart the runtime to use the right version of `pyarrow`."
     )
 
-from .arrow_dataset import Dataset, concatenate_datasets
-from .arrow_reader import ArrowReader, ReadInstruction
-from .arrow_writer import ArrowWriter
+SCRIPTS_VERSION = "main" if version.parse(__version__).is_devrelease else __version__
+
+del pyarrow
+del version
+
+from .arrow_dataset import Dataset
+from .arrow_reader import ReadInstruction
 from .builder import ArrowBasedBuilder, BeamBasedBuilder, BuilderConfig, DatasetBuilder, GeneratorBasedBuilder
-from .combine import interleave_datasets
+from .combine import concatenate_datasets, interleave_datasets
 from .dataset_dict import DatasetDict, IterableDatasetDict
-from .features import (
-    Array2D,
-    Array3D,
-    Array4D,
-    Array5D,
-    Audio,
-    ClassLabel,
-    Features,
-    Image,
-    Sequence,
-    Translation,
-    TranslationVariableLanguages,
-    Value,
-)
-from .fingerprint import is_caching_enabled, set_caching_enabled
+from .download import *
+from .features import *
+from .fingerprint import disable_caching, enable_caching, is_caching_enabled, set_caching_enabled
 from .info import DatasetInfo, MetricInfo
 from .inspect import (
+    get_dataset_config_info,
     get_dataset_config_names,
     get_dataset_infos,
     get_dataset_split_names,
@@ -63,8 +54,7 @@ from .inspect import (
     list_metrics,
 )
 from .iterable_dataset import IterableDataset
-from .keyhash import KeyHasher
-from .load import import_main_class, load_dataset, load_dataset_builder, load_from_disk, load_metric, prepare_module
+from .load import load_dataset, load_dataset_builder, load_from_disk, load_metric
 from .metric import Metric
 from .splits import (
     NamedSplit,
@@ -79,6 +69,20 @@ from .splits import (
 )
 from .tasks import *
 from .utils import *
+from .utils import logging
 
 
-SCRIPTS_VERSION = "master" if _version.parse(__version__).is_devrelease else __version__
+# deprecated modules
+from datasets import arrow_dataset as _arrow_dataset  # isort:skip
+from datasets import utils as _utils  # isort:skip
+from datasets.utils import download_manager as _deprecated_download_manager  # isort:skip
+
+_arrow_dataset.concatenate_datasets = concatenate_datasets
+_utils.DownloadConfig = DownloadConfig
+_utils.DownloadManager = DownloadManager
+_utils.DownloadMode = DownloadMode
+_deprecated_download_manager.DownloadConfig = DownloadConfig
+_deprecated_download_manager.DownloadMode = DownloadMode
+_deprecated_download_manager.DownloadManager = DownloadManager
+
+del _arrow_dataset, _utils, _deprecated_download_manager
