@@ -7,8 +7,8 @@ from typing import Generator
 import datasets.config
 from datasets.builder import DatasetBuilder
 from datasets.commands import BaseDatasetsCLICommand
+from datasets.download.download_manager import DownloadMode
 from datasets.load import dataset_module_factory, import_main_class
-from datasets.utils.download_manager import GenerateMode
 from datasets.utils.filelock import logger as fl_logger
 from datasets.utils.logging import ERROR, get_logger
 
@@ -16,7 +16,7 @@ from datasets.utils.logging import ERROR, get_logger
 logger = get_logger(__name__)
 
 
-def test_command_factory(args):
+def _test_command_factory(args):
     return TestCommand(
         args.dataset,
         args.name,
@@ -27,12 +27,12 @@ def test_command_factory(args):
         args.ignore_verifications,
         args.force_redownload,
         args.clear_cache,
-        args.proc_rank,
-        args.num_proc,
     )
 
 
 class TestCommand(BaseDatasetsCLICommand):
+    __test__ = False  # to tell pytest it's not a test class
+
     @staticmethod
     def register_subcommand(parser: ArgumentParser):
         test_parser = parser.add_parser("test", help="Test dataset implementation.")
@@ -60,20 +60,8 @@ class TestCommand(BaseDatasetsCLICommand):
             action="store_true",
             help="Remove downloaded files and cached datasets after each config test",
         )
-        test_parser.add_argument(
-            "--proc_rank",
-            type=int,
-            default=0,
-            help="Rank of the current process for multiprocessing testing.",
-        )
-        test_parser.add_argument(
-            "--num_proc",
-            type=int,
-            default=1,
-            help="Number of processes to use for multiprocessing testing",
-        )
         test_parser.add_argument("dataset", type=str, help="Name of the dataset to download")
-        test_parser.set_defaults(func=test_command_factory)
+        test_parser.set_defaults(func=_test_command_factory)
 
     def __init__(
         self,
@@ -86,8 +74,6 @@ class TestCommand(BaseDatasetsCLICommand):
         ignore_verifications: bool,
         force_redownload: bool,
         clear_cache: bool,
-        proc_rank: int,
-        num_proc: int,
     ):
         self._dataset = dataset
         self._name = name
@@ -98,8 +84,6 @@ class TestCommand(BaseDatasetsCLICommand):
         self._ignore_verifications = ignore_verifications
         self._force_redownload = force_redownload
         self._clear_cache = clear_cache
-        self._proc_rank = proc_rank
-        self._num_proc = num_proc
         if clear_cache and not cache_dir:
             print(
                 "When --clear_cache is used, specifying a cache directory is mandatory.\n"
@@ -115,38 +99,45 @@ class TestCommand(BaseDatasetsCLICommand):
         if self._name is not None and self._all_configs:
             print("Both parameters `config` and `all_configs` can't be used at once.")
             exit(1)
-        path, name = self._dataset, self._name
+        path, config_name = self._dataset, self._name
         module = dataset_module_factory(path)
         builder_cls = import_main_class(module.module_path)
-
-        if self._all_configs and len(builder_cls.BUILDER_CONFIGS) > 0:
-            n_builders = len(builder_cls.BUILDER_CONFIGS) // self._num_proc
-            n_builders += (len(builder_cls.BUILDER_CONFIGS) % self._num_proc) > self._proc_rank
-        else:
-            n_builders = 1 if self._proc_rank == 0 else 0
+        n_builders = len(builder_cls.BUILDER_CONFIGS) if self._all_configs and builder_cls.BUILDER_CONFIGS else 1
 
         def get_builders() -> Generator[DatasetBuilder, None, None]:
-            if self._all_configs and len(builder_cls.BUILDER_CONFIGS) > 0:
+            if self._all_configs and builder_cls.BUILDER_CONFIGS:
                 for i, config in enumerate(builder_cls.BUILDER_CONFIGS):
-                    if i % self._num_proc == self._proc_rank:
+                    if "config_name" in module.builder_kwargs:
                         yield builder_cls(
-                            name=config.name,
+                            cache_dir=self._cache_dir,
+                            data_dir=self._data_dir,
+                            **module.builder_kwargs,
+                        )
+                    else:
+                        yield builder_cls(
+                            config_name=config.name,
                             cache_dir=self._cache_dir,
                             data_dir=self._data_dir,
                             **module.builder_kwargs,
                         )
             else:
-                if self._proc_rank == 0:
+                if "config_name" in module.builder_kwargs:
+                    yield builder_cls(cache_dir=self._cache_dir, data_dir=self._data_dir, **module.builder_kwargs)
+                else:
                     yield builder_cls(
-                        name=name, cache_dir=self._cache_dir, data_dir=self._data_dir, **module.builder_kwargs
+                        config_name=config_name,
+                        cache_dir=self._cache_dir,
+                        data_dir=self._data_dir,
+                        **module.builder_kwargs,
                     )
 
         for j, builder in enumerate(get_builders()):
             print(f"Testing builder '{builder.config.name}' ({j + 1}/{n_builders})")
+            builder._record_infos = True
             builder.download_and_prepare(
-                download_mode=GenerateMode.REUSE_CACHE_IF_EXISTS
+                download_mode=DownloadMode.REUSE_CACHE_IF_EXISTS
                 if not self._force_redownload
-                else GenerateMode.FORCE_REDOWNLOAD,
+                else DownloadMode.FORCE_REDOWNLOAD,
                 ignore_verifications=self._ignore_verifications,
                 try_from_hf_gcs=False,
             )
@@ -167,15 +158,17 @@ class TestCommand(BaseDatasetsCLICommand):
                     dataset_dir = os.path.dirname(path)
                 elif os.path.isfile(combined_path):
                     dataset_dir = path
+                elif os.path.isdir(path):  # for local directories containing only data files
+                    dataset_dir = path
                 else:  # in case of a remote dataset
                     dataset_dir = None
-                    print("Dataset Infos file saved at {}".format(dataset_infos_path))
+                    print(f"Dataset Infos file saved at {dataset_infos_path}")
 
                 # Move dataset_info back to the user
                 if dataset_dir is not None:
                     user_dataset_infos_path = os.path.join(dataset_dir, datasets.config.DATASETDICT_INFOS_FILENAME)
                     copyfile(dataset_infos_path, user_dataset_infos_path)
-                    print("Dataset Infos file saved at {}".format(user_dataset_infos_path))
+                    print(f"Dataset Infos file saved at {user_dataset_infos_path}")
 
             # If clear_cache=True, the download folder and the dataset builder cache directory are deleted
             if self._clear_cache:
