@@ -1,4 +1,3 @@
-# coding=utf-8
 # Copyright 2020 The HuggingFace Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -28,7 +27,7 @@ if TYPE_CHECKING:
 
 
 class TorchFormatter(Formatter[dict, "torch.Tensor", dict]):
-    def __init__(self, **torch_tensor_kwargs):
+    def __init__(self, features=None, decoded=True, **torch_tensor_kwargs):
         self.torch_tensor_kwargs = torch_tensor_kwargs
         import torch  # noqa import torch at initialization
 
@@ -47,7 +46,7 @@ class TorchFormatter(Formatter[dict, "torch.Tensor", dict]):
         # support for nested types like struct of list of struct
         if isinstance(data_struct, (list, np.ndarray)):
             data_struct = np.array(data_struct, copy=False)
-            if data_struct.dtype == np.object:  # pytorch tensors cannot be instantied from an array of objects
+            if data_struct.dtype == object:  # pytorch tensors cannot be instantied from an array of objects
                 return [self.recursive_tensorize(substruct) for substruct in data_struct]
         return self._tensorize(data_struct)
 
