@@ -1,5 +1,6 @@
+import copy
 from dataclasses import dataclass
-from typing import ClassVar, Dict, Optional, Tuple
+from typing import ClassVar, Dict
 
 from ..features import ClassLabel, Features, Value
 from .base import TaskTemplate
@@ -10,19 +11,20 @@ class TextClassification(TaskTemplate):
     # `task` is not a ClassVar since we want it to be part of the `asdict` output for JSON serialization
     task: str = "text-classification"
     input_schema: ClassVar[Features] = Features({"text": Value("string")})
-    # TODO(lewtun): Find a more elegant approach without descriptors.
     label_schema: ClassVar[Features] = Features({"labels": ClassLabel})
     text_column: str = "text"
     label_column: str = "labels"
-    labels: Optional[Tuple[str]] = None
 
-    def __post_init__(self):
-        if self.labels:
-            assert len(self.labels) == len(set(self.labels)), "Labels must be unique"
-            # Cast labels to tuple to allow hashing
-            self.__dict__["labels"] = tuple(sorted(self.labels))
-            self.__dict__["label_schema"] = self.label_schema.copy()
-            self.label_schema["labels"] = ClassLabel(names=self.labels)
+    def align_with_features(self, features):
+        if self.label_column not in features:
+            raise ValueError(f"Column {self.label_column} is not present in features.")
+        if not isinstance(features[self.label_column], ClassLabel):
+            raise ValueError(f"Column {self.label_column} is not a ClassLabel.")
+        task_template = copy.deepcopy(self)
+        label_schema = self.label_schema.copy()
+        label_schema["labels"] = features[self.label_column]
+        task_template.__dict__["label_schema"] = label_schema
+        return task_template
 
     @property
     def column_mapping(self) -> Dict[str, str]:
