@@ -15,19 +15,17 @@ import sys
 import tempfile
 import time
 import urllib
-import warnings
 from contextlib import closing, contextmanager
-from dataclasses import dataclass
 from functools import partial
 from hashlib import sha256
 from pathlib import Path
-from typing import Dict, Optional, TypeVar, Union
+from typing import List, Optional, Type, TypeVar, Union
 from urllib.parse import urljoin, urlparse
 
-import numpy as np
 import requests
 
-from .. import __version__, config, utils
+from .. import __version__, config
+from ..download.download_config import DownloadConfig
 from . import logging
 from .extract import ExtractManager
 from .filelock import FileLock
@@ -59,60 +57,6 @@ def init_hf_modules(hf_modules_cache: Optional[Union[Path, str]] = None) -> str:
     return hf_modules_cache
 
 
-@contextmanager
-def temp_seed(seed: int, set_pytorch=False, set_tensorflow=False):
-    """Temporarily set the random seed. This works for python numpy, pytorch and tensorflow."""
-    np_state = np.random.get_state()
-    np.random.seed(seed)
-
-    if set_pytorch and config.TORCH_AVAILABLE:
-        import torch
-
-        torch_state = torch.random.get_rng_state()
-        torch.random.manual_seed(seed)
-
-        if torch.cuda.is_available():
-            torch_cuda_states = torch.cuda.get_rng_state_all()
-            torch.cuda.manual_seed_all(seed)
-
-    if set_tensorflow and config.TF_AVAILABLE:
-        import tensorflow as tf
-        from tensorflow.python import context as tfpycontext
-
-        tf_state = tf.random.get_global_generator()
-        temp_gen = tf.random.Generator.from_seed(seed)
-        tf.random.set_global_generator(temp_gen)
-
-        if not tf.executing_eagerly():
-            raise ValueError("Setting random seed for TensorFlow is only available in eager mode")
-
-        tf_context = tfpycontext.context()  # eager mode context
-        tf_seed = tf_context._seed
-        tf_rng_initialized = hasattr(tf_context, "_rng")
-        if tf_rng_initialized:
-            tf_rng = tf_context._rng
-        tf_context._set_global_seed(seed)
-
-    try:
-        yield
-    finally:
-        np.random.set_state(np_state)
-
-        if set_pytorch and config.TORCH_AVAILABLE:
-            torch.random.set_rng_state(torch_state)
-            if torch.cuda.is_available():
-                torch.cuda.set_rng_state_all(torch_cuda_states)
-
-        if set_tensorflow and config.TF_AVAILABLE:
-            tf.random.set_global_generator(tf_state)
-
-            tf_context._seed = tf_seed
-            if tf_rng_initialized:
-                tf_context._rng = tf_rng
-            else:
-                delattr(tf_context, "_rng")
-
-
 def is_remote_url(url_or_filename: str) -> bool:
     parsed = urlparse(url_or_filename)
     return parsed.scheme in ("http", "https", "s3", "gs", "hdfs", "ftp")
@@ -152,14 +96,9 @@ def head_hf_s3(
     )
 
 
-def hf_github_url(path: str, name: str, dataset=True, revision: Optional[str] = None, version="deprecated") -> str:
+def hf_github_url(path: str, name: str, dataset=True, revision: Optional[str] = None) -> str:
     from .. import SCRIPTS_VERSION
 
-    if version != "deprecated":
-        warnings.warn(
-            "'version' was renamed to 'revision' in version 1.13 and will be removed in 1.15.", FutureWarning
-        )
-        revision = version
     revision = revision or os.getenv("HF_SCRIPTS_VERSION", SCRIPTS_VERSION)
     if dataset:
         return config.REPO_DATASETS_URL.format(revision=revision, path=path, name=name)
@@ -167,14 +106,9 @@ def hf_github_url(path: str, name: str, dataset=True, revision: Optional[str] =
         return config.REPO_METRICS_URL.format(revision=revision, path=path, name=name)
 
 
-def hf_hub_url(path: str, name: str, revision: Optional[str] = None, version="deprecated") -> str:
-    if version != "deprecated":
-        warnings.warn(
-            "'version' was renamed to 'revision' in version 1.13 and will be removed in 1.15.", FutureWarning
-        )
-        revision = version
+def hf_hub_url(repo_id: str, path: str, revision: Optional[str] = None) -> str:
     revision = revision or config.HUB_DEFAULT_VERSION
-    return config.HUB_DATASETS_URL.format(path=path, name=name, revision=revision)
+    return config.HUB_DATASETS_URL.format(repo_id=repo_id, path=path, revision=revision)
 
 
 def url_or_path_join(base_name: str, *pathnames: str) -> str:
@@ -215,53 +149,6 @@ def hash_url_to_filename(url, etag=None):
     return filename
 
 
-@dataclass
-class DownloadConfig:
-    """Configuration for our cached path manager.
-
-    Attributes:
-        cache_dir (:obj:`str` or :obj:`Path`, optional): Specify a cache directory to save the file to (overwrite the
-            default cache dir).
-        force_download (:obj:`bool`, default ``False``): If True, re-dowload the file even if it's already cached in
-            the cache dir.
-        resume_download (:obj:`bool`, default ``False``): If True, resume the download if incompletly recieved file is
-            found.
-        proxies (:obj:`dict`, optional):
-        user_agent (:obj:`str`, optional): Optional string or dict that will be appended to the user-agent on remote
-            requests.
-        extract_compressed_file (:obj:`bool`, default ``False``): If True and the path point to a zip or tar file,
-            extract the compressed file in a folder along the archive.
-        force_extract (:obj:`bool`, default ``False``): If True when extract_compressed_file is True and the archive
-            was already extracted, re-extract the archive and override the folder where it was extracted.
-        delete_extracted (:obj:`bool`, default ``False``): Whether to delete (or keep) the extracted files.
-        use_etag (:obj:`bool`, default ``True``): Whether to use the ETag HTTP response header to validate the cached files.
-        num_proc (:obj:`int`, optional): The number of processes to launch to download the files in parallel.
-        max_retries (:obj:`int`, default ``1``): The number of times to retry an HTTP request if it fails.
-        use_auth_token (:obj:`str` or :obj:`bool`, optional): Optional string or boolean to use as Bearer token
-            for remote files on the Datasets Hub. If True, will get token from ~/.huggingface.
-        ignore_url_params (:obj:`bool`, default ``False``): Whether to strip all query parameters and #fragments from
-            the download URL before using it for caching the file.
-    """
-
-    cache_dir: Optional[Union[str, Path]] = None
-    force_download: bool = False
-    resume_download: bool = False
-    local_files_only: bool = False
-    proxies: Optional[Dict] = None
-    user_agent: Optional[str] = None
-    extract_compressed_file: bool = False
-    force_extract: bool = False
-    delete_extracted: bool = False
-    use_etag: bool = True
-    num_proc: Optional[int] = None
-    max_retries: int = 1
-    use_auth_token: Optional[Union[str, bool]] = None
-    ignore_url_params: bool = False
-
-    def copy(self) -> "DownloadConfig":
-        return self.__class__(**{k: copy.deepcopy(v) for k, v in self.__dict__.items()})
-
-
 def cached_path(
     url_or_filename,
     download_config=None,
@@ -307,6 +194,7 @@ def cached_path(
             max_retries=download_config.max_retries,
             use_auth_token=download_config.use_auth_token,
             ignore_url_params=download_config.ignore_url_params,
+            download_desc=download_config.download_desc,
         )
     elif os.path.exists(url_or_filename):
         # File, and it exists.
@@ -375,6 +263,32 @@ def _raise_if_offline_mode_is_enabled(msg: Optional[str] = None):
         )
 
 
+def _retry(
+    func,
+    func_args: Optional[tuple] = None,
+    func_kwargs: Optional[dict] = None,
+    exceptions: Type[requests.exceptions.RequestException] = requests.exceptions.RequestException,
+    status_codes: Optional[List[int]] = None,
+    max_retries: int = 0,
+    base_wait_time: float = 0.5,
+    max_wait_time: float = 2,
+):
+    func_args = func_args or ()
+    func_kwargs = func_kwargs or {}
+    retry = 0
+    while True:
+        try:
+            return func(*func_args, **func_kwargs)
+        except exceptions as err:
+            if retry >= max_retries or (status_codes and err.response.status_code not in status_codes):
+                raise err
+            else:
+                sleep_time = min(max_wait_time, base_wait_time * 2**retry)  # Exponential backoff
+                logger.info(f"{func} timed out, retrying in {sleep_time}s... [{retry/max_retries}]")
+                time.sleep(sleep_time)
+                retry += 1
+
+
 def _request_with_retry(
     method: str,
     url: str,
@@ -395,7 +309,7 @@ def _request_with_retry(
         base_wait_time (float): Duration (in seconds) to wait before retrying the first time. Wait time between
             retries then grows exponentially, capped by max_wait_time.
         max_wait_time (float): Maximum amount of time between two retries, in seconds.
-        **params: Params to pass to :obj:`requests.request`.
+        **params (additional keyword arguments): Params to pass to :obj:`requests.request`.
     """
     _raise_if_offline_mode_is_enabled(f"Tried to reach {url}")
     tries, success = 0, False
@@ -434,7 +348,9 @@ def ftp_get(url, temp_file, timeout=10.0):
         raise ConnectionError(e) from None
 
 
-def http_get(url, temp_file, proxies=None, resume_size=0, headers=None, cookies=None, timeout=100.0, max_retries=0):
+def http_get(
+    url, temp_file, proxies=None, resume_size=0, headers=None, cookies=None, timeout=100.0, max_retries=0, desc=None
+):
     headers = copy.deepcopy(headers) or {}
     headers["user-agent"] = get_datasets_user_agent(user_agent=headers.get("user-agent"))
     if resume_size > 0:
@@ -453,19 +369,17 @@ def http_get(url, temp_file, proxies=None, resume_size=0, headers=None, cookies=
         return
     content_length = response.headers.get("Content-Length")
     total = resume_size + int(content_length) if content_length is not None else None
-    progress = utils.tqdm(
+    with logging.tqdm(
         unit="B",
         unit_scale=True,
         total=total,
         initial=resume_size,
-        desc="Downloading",
-        disable=bool(logging.get_verbosity() == logging.NOTSET),
-    )
-    for chunk in response.iter_content(chunk_size=1024):
-        if chunk:  # filter out keep-alive new chunks
+        desc=desc or "Downloading",
+        disable=not logging.is_progress_bar_enabled(),
+    ) as progress:
+        for chunk in response.iter_content(chunk_size=1024):
             progress.update(len(chunk))
             temp_file.write(chunk)
-    progress.close()
 
 
 def http_head(
@@ -507,6 +421,7 @@ def get_from_cache(
     max_retries=0,
     use_auth_token=None,
     ignore_url_params=False,
+    download_desc=None,
 ) -> str:
     """
     Given a URL, look for the corresponding file in the local cache.
@@ -574,6 +489,9 @@ def get_from_cache(
                         url += "&confirm=" + v
                         cookies = response.cookies
                 connected = True
+                # Fix Google Drive URL to avoid Virus scan warning
+                if "drive.google.com" in url and "confirm=" not in url:
+                    url += "&confirm=t"
             # In some edge cases, head request returns 400 but the connection is actually ok
             elif (
                 (response.status_code == 400 and "firebasestorage.googleapis.com" in url)
@@ -591,9 +509,9 @@ def get_from_cache(
                 logger.info(f"Couldn't get ETag version for url {url}")
             elif response.status_code == 401 and config.HF_ENDPOINT in url and use_auth_token is None:
                 raise ConnectionError(
-                    f"Unauthorized for URL {url}. Please use the parameter ``use_auth_token=True`` after logging in with ``huggingface-cli login``"
+                    f"Unauthorized for URL {url}. Please use the parameter `use_auth_token=True` after logging in with `huggingface-cli login`"
                 )
-        except (EnvironmentError, requests.exceptions.Timeout) as e:
+        except (OSError, requests.exceptions.Timeout) as e:
             # not connected
             head_error = e
             pass
@@ -664,6 +582,7 @@ def get_from_cache(
                     headers=headers,
                     cookies=cookies,
                     max_retries=max_retries,
+                    desc=download_desc,
                 )
 
         logger.info(f"storing {url} in cache at {cache_path}")
