@@ -1,10 +1,19 @@
 import json
 import logging
 import re
+import warnings
 from collections import Counter
-from dataclasses import asdict, dataclass, fields
+from dataclasses import dataclass, fields
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
+from typing import Any, Callable, ClassVar, Dict, List, Optional, Tuple, Type, Union
+
+
+try:  # Python >= 3.8
+    from typing import get_args
+except ImportError:
+
+    def get_args(tp):
+        return tp.__args__
 
 
 # loading package files: https://stackoverflow.com/a/20885799
@@ -19,7 +28,7 @@ import yaml
 from . import resources
 
 
-BASE_REF_URL = "https://github.com/huggingface/datasets/tree/master/src/datasets/utils"
+BASE_REF_URL = "https://github.com/huggingface/datasets/tree/main/src/datasets/utils"
 this_url = f"{BASE_REF_URL}/{__file__}"
 logger = logging.getLogger(__name__)
 
@@ -29,11 +38,20 @@ def load_json_resource(resource: str) -> Tuple[Any, str]:
     return json.loads(content), f"{BASE_REF_URL}/resources/{resource}"
 
 
+def load_tsv_licenses(resource: str) -> Tuple[Any, str]:
+    content = pkg_resources.read_text(resources, resource)
+    licenses = {
+        line.split("\t")[1].strip(): line.split("\t")[0].strip() for line in content.splitlines() if "\t" in line
+    }
+    return licenses, f"{BASE_REF_URL}/resources/{resource}"
+
+
 # Source of languages.json:
 # https://datahub.io/core/language-codes/r/ietf-language-tags.csv
 # Language names were obtained with langcodes: https://github.com/LuminosoInsight/langcodes
 known_language_codes, known_language_codes_url = load_json_resource("languages.json")
-known_licenses, known_licenses_url = load_json_resource("licenses.json")
+# standard_licenses.tsv is to be kept in sync with the same file in `moon-landing` and `hub-docs`
+known_licenses, known_licenses_url = load_tsv_licenses("standard_licenses.tsv")
 known_task_ids, known_task_ids_url = load_json_resource("tasks.json")
 known_creators, known_creators_url = load_json_resource("creators.json")
 known_size_categories, known_size_categories_url = load_json_resource("size_categories.json")
@@ -114,12 +132,12 @@ def tagset_validator(
 def validate_type(value: Any, expected_type: Type):
     error_string = ""
     NoneType = type(None)
-    if expected_type == NoneType:
+    if expected_type is NoneType:
         if not isinstance(value, NoneType):
             return f"Expected `{NoneType}`. Found value: `{value}` of type `{type(value)}`.\n"
         else:
             return error_string
-    if expected_type == str:
+    if expected_type is str:
         if not isinstance(value, str):
             return f"Expected `{str}`. Found value: `{value}` of type: `{type(value)}`.\n"
 
@@ -131,10 +149,10 @@ def validate_type(value: Any, expected_type: Type):
             return error_string
     # Add more `elif` statements if primitive type checking is needed
     else:
-        expected_type_origin = expected_type.__origin__
-        expected_type_args = expected_type.__args__
+        expected_type_name = str(expected_type).split(".", 1)[-1].split("[")[0]  # typing.List[str] -> List
+        expected_type_args = get_args(expected_type)
 
-        if expected_type_origin == Union:
+        if expected_type_name == "Union":
             for type_arg in expected_type_args:
                 temp_error_string = validate_type(value, type_arg)
                 if temp_error_string == "":  # at least one type is successfully validated
@@ -144,29 +162,43 @@ def validate_type(value: Any, expected_type: Type):
                         error_string = "(" + temp_error_string + ")"
                     else:
                         error_string += "\nOR\n" + "(" + temp_error_string + ")"
-
+        elif value is None:
+            return f"Expected `{expected_type}`. Found value: `{value}` of type: `{type(value)}`.\n"
         else:
-            # Assuming `List`/`Dict`/`Tuple`
-            if not isinstance(value, expected_type_origin) or len(value) == 0:
-                return f"Expected `{expected_type_origin}` with length > 0. Found value of type: `{type(value)}`, with length: {len(value)}.\n"
+            # Assuming non empty `List`/`Dict`/`Tuple`
+            if expected_type is EmptyList:
+                if len(value) == 0:
+                    return ""
+                else:
+                    return f"Expected `{expected_type}` of length 0. Found value of type: `{type(value)}`, with length: {len(value)}.\n"
 
-            if expected_type_origin == Dict:
-                key_type, value_type = expected_type_args
-                key_error_string = ""
-                value_error_string = ""
-                for k, v in value.items():
-                    key_error_string += validate_type(k, key_type)
-                    value_error_string += validate_type(v, value_type)
-                if key_error_string != "" or value_error_string != "":
-                    return f"Typing errors with keys:\n {key_error_string} and values:\n {value_error_string}"
+            # Assuming non empty
+            if not isinstance(value, (dict, tuple, list)) or len(value) == 0:
+                return f"Expected `{expected_type}` with length > 0. Found value of type: `{type(value)}`, with length: {len(value)}.\n"
+
+            if expected_type_name == "Dict":
+                if not isinstance(value, dict):
+                    return f"Expected `{expected_type}` with length > 0. Found value of type: `{type(value)}`, with length: {len(value)}.\n"
+                if expected_type_args != get_args(Dict):  # if we specified types for keys and values
+                    key_type, value_type = expected_type_args
+                    key_error_string = ""
+                    value_error_string = ""
+                    for k, v in value.items():
+                        key_error_string += validate_type(k, key_type)
+                        value_error_string += validate_type(v, value_type)
+                    if key_error_string != "" or value_error_string != "":
+                        return f"Typing errors with keys:\n {key_error_string} and values:\n {value_error_string}"
 
             else:  # `List`/`Tuple`
-                value_type = expected_type_args[0]
-                value_error_string = ""
-                for v in value:
-                    value_error_string += validate_type(v, value_type)
-                if value_error_string != "":
-                    return f"Typing errors with values:\n {value_error_string}"
+                if not isinstance(value, (list, tuple)):
+                    return f"Expected `{expected_type}` with length > 0. Found value of type: `{type(value)}`, with length: {len(value)}.\n"
+                if expected_type_args != get_args(List):  # if we specified types for the items in the list
+                    value_type = expected_type_args[0]
+                    value_error_string = ""
+                    for v in value:
+                        value_error_string += validate_type(v, value_type)
+                    if value_error_string != "":
+                        return f"Typing errors with values:\n {value_error_string}"
 
         return error_string
 
@@ -185,19 +217,45 @@ def validate_metadata_type(metadata_dict: dict):
         raise TypeError(f"The following typing errors are found: {typing_errors}")
 
 
+class _nothing:
+    pass
+
+
+EmptyList = List[_nothing]
+
+
 @dataclass
 class DatasetMetadata:
-    annotations_creators: Union[List[str], Dict[str, List[str]]]
-    language_creators: Union[List[str], Dict[str, List[str]]]
-    languages: Union[List[str], Dict[str, List[str]]]
-    licenses: Union[List[str], Dict[str, List[str]]]
-    multilinguality: Union[List[str], Dict[str, List[str]]]
-    pretty_name: Union[str, Dict[str, str]]
-    size_categories: Union[List[str], Dict[str, List[str]]]
-    source_datasets: Union[List[str], Dict[str, List[str]]]
-    task_categories: Union[List[str], Dict[str, List[str]]]
-    task_ids: Union[List[str], Dict[str, List[str]]]
+    annotations_creators: List[str]
+    language_creators: Union[EmptyList, List[str]]
+    language: Union[EmptyList, List[str]]
+    license: List[str]
+    multilinguality: List[str]
+    pretty_name: str
+    size_categories: List[str]
+    source_datasets: List[str]
+    task_categories: List[str]
+    task_ids: Union[EmptyList, List[str]]
     paperswithcode_id: Optional[str] = None
+    train_eval_index: Optional[List[Dict]] = None
+    configs: Optional[List[str]] = None
+    extra_gated_fields: Optional[Dict] = None
+    extra_gated_prompt: Optional[str] = None
+    license_details: Optional[str] = None
+    tags: Optional[List[str]] = None
+    licenses: Optional[Union[EmptyList, List[str]]] = None  # deprecated
+    languages: Optional[Union[EmptyList, List[str]]] = None  # deprecated
+
+    # class attributes
+    _FIELDS_WITH_DASHES: ClassVar[set] = {"train_eval_index"}  # train-eval-index in the YAML metadata
+    _ALLOWED_YAML_KEYS: ClassVar[set] = set()  # populated later
+    _DEPRECATED_YAML_KEYS = ["licenses", "languages"]
+
+    def __post_init__(self):
+        if self.licenses is not None:
+            warnings.warning("The 'licenses' YAML field is deprecated, please use 'license' instead.")
+        if self.languages is not None:
+            warnings.warning("The 'languages' YAML field is deprecated, please use 'language' instead.")
 
     def validate(self):
         validate_metadata_type(metadata_dict=vars(self))
@@ -206,8 +264,8 @@ class DatasetMetadata:
             self.annotations_creators
         )
         self.language_creators, language_creators_errors = self.validate_language_creators(self.language_creators)
-        self.languages, languages_errors = self.validate_language_codes(self.languages)
-        self.licenses, licenses_errors = self.validate_licences(self.licenses)
+        self.language, language_errors = self.validate_language_codes(self.language or self.languages)
+        self.license, license_errors = self.validate_licences(self.license or self.licenses)
         self.multilinguality, multilinguality_errors = self.validate_mulitlinguality(self.multilinguality)
         self.size_categories, size_categories_errors = self.validate_size_catgeories(self.size_categories)
         self.source_datasets, source_datasets_errors = self.validate_source_datasets(self.source_datasets)
@@ -216,24 +274,26 @@ class DatasetMetadata:
         self.paperswithcode_id, paperswithcode_id_errors = self.validate_paperswithcode_id_errors(
             self.paperswithcode_id
         )
+        self.train_eval_index, train_eval_index_errors = self.validate_train_eval_index(self.train_eval_index)
 
         errors = {
             "annotations_creators": annotations_creators_errors,
             "language_creators": language_creators_errors,
-            "licenses": licenses_errors,
+            "license": license_errors,
             "multilinguality": multilinguality_errors,
             "size_categories": size_categories_errors,
             "source_datasets": source_datasets_errors,
             "task_categories": task_categories_errors,
             "task_ids": task_ids_errors,
-            "languages": languages_errors,
+            "language": language_errors,
             "paperswithcode_id": paperswithcode_id_errors,
+            "train_eval_index": train_eval_index_errors,
         }
 
         exception_msg_dict = dict()
-        for field, errs in errors.items():
+        for yaml_field, errs in errors.items():
             if errs is not None:
-                exception_msg_dict[field] = errs
+                exception_msg_dict[yaml_field] = errs
         if len(exception_msg_dict) > 0:
             raise TypeError(
                 "Could not validate the metadata, found the following errors:\n"
@@ -261,9 +321,41 @@ class DatasetMetadata:
             raise TypeError(f"Unable to find a yaml block in '{path}'")
 
     @classmethod
-    def from_yaml_string(cls, string: str) -> "DatasetMetadata":
+    def _metadata_dict_from_yaml_string(cls, string: str) -> dict:
         """Loads and validates the dataset metadat from a YAML string
 
+        Args:
+            string (:obj:`str`): The YAML string
+
+        Returns:
+            :class:`dict`: The dataset's metadata as a dictionary
+
+        Raises:
+            :obj:`TypeError`: If the dataset's metadata is invalid
+        """
+        metadata_dict = yaml.load(string, Loader=NoDuplicateSafeLoader) or dict()
+
+        # Check if the YAML keys are all correct
+        bad_keys = [k for k in metadata_dict if k not in cls._ALLOWED_YAML_KEYS]
+        if bad_keys:
+            raise ValueError(f"Bad YAML keys: {bad_keys}. Allowed fields: {cls._ALLOWED_YAML_KEYS}")
+
+        # Check if config names are valid
+        bad_keys = [k for k in metadata_dict if k not in cls._ALLOWED_YAML_KEYS]
+        if bad_keys:
+            raise ValueError(f"Bad YAML keys: {bad_keys}. Allowed fields: {cls._ALLOWED_YAML_KEYS}")
+
+        # Convert the YAML keys to DatasetMetadata fields
+        metadata_dict = {
+            (key.replace("-", "_") if key.replace("-", "_") in cls._FIELDS_WITH_DASHES else key): value
+            for key, value in metadata_dict.items()
+        }
+        return metadata_dict
+
+    @classmethod
+    def from_yaml_string(cls, string: str) -> "DatasetMetadata":
+        """Loads and validates the dataset metadata from a YAML string
+
         Args:
             string (:obj:`str`): The YAML string
 
@@ -272,9 +364,10 @@ class DatasetMetadata:
 
         Raises:
             :obj:`TypeError`: If the dataset's metadata is invalid
+            :obj:`ValueError`: If the dataset's metadata is invalid
         """
-        metada_dict = yaml.load(string, Loader=NoDuplicateSafeLoader) or dict()
-        return cls(**metada_dict)
+        metadata_dict = cls._metadata_dict_from_yaml_string(string)
+        return cls(**metadata_dict)
 
     @staticmethod
     def validate_annotations_creators(annotations_creators: Union[List[str], Dict[str, List[str]]]) -> ValidatorOutput:
@@ -291,7 +384,7 @@ class DatasetMetadata:
         return tagset_validator(
             languages,
             known_language_codes.keys(),
-            "languages",
+            "language",
             known_language_codes_url,
             lambda lang: lang == "unknown",
         )
@@ -301,9 +394,8 @@ class DatasetMetadata:
         validated, error = tagset_validator(
             licenses,
             list(known_licenses.keys()),
-            "licenses",
+            "license",
             known_licenses_url,
-            lambda e: "-other-" in e or e.startswith("other-"),
         )
         return validated, error
 
@@ -321,9 +413,13 @@ class DatasetMetadata:
     def validate_task_ids(task_ids: Union[List[str], Dict[str, List[str]]]) -> ValidatorOutput:
         # TODO: we're currently ignoring all values starting with 'other' as our task taxonomy is bound to change
         #   in the near future and we don't want to waste energy in tagging against a moving taxonomy.
-        known_set = [tid for _cat, d in known_task_ids.items() for tid in d["options"]]
+        known_set = [tid for _cat, d in known_task_ids.items() for tid in d.get("subtasks", [])]
         validated, error = tagset_validator(
-            task_ids, known_set, "task_ids", known_task_ids_url, lambda e: "-other-" in e or e.startswith("other-")
+            task_ids,
+            known_set,
+            "task_ids",
+            known_task_ids_url,
+            lambda e: not e or "-other-" in e or e.startswith("other-"),
         )
         return validated, error
 
@@ -375,31 +471,22 @@ class DatasetMetadata:
             else:
                 return pretty_name, None
 
-    def get_metadata_by_config_name(self, name: str) -> "DatasetMetadata":
-        metadata_dict = asdict(self)
-        config_name_hit = []
-        has_multi_configs = []
-        result_dict = {}
-        for tag_key, tag_value in metadata_dict.items():
-            if isinstance(tag_value, str) or isinstance(tag_value, list):
-                result_dict[tag_key] = tag_value
-            elif isinstance(tag_value, dict):
-                has_multi_configs.append(tag_key)
-                for config_name, value in tag_value.items():
-                    if config_name == name:
-                        result_dict[tag_key] = value
-                        config_name_hit.append(tag_key)
+    @staticmethod
+    def validate_train_eval_index(train_eval_index: Optional[Dict]):
+        if train_eval_index is not None and not isinstance(train_eval_index, list):
+            return None, f"train-eval-index must be a list, but got {type(train_eval_index)} instead."
+        else:
+            return train_eval_index, None
 
-        if len(has_multi_configs) > 0 and has_multi_configs != config_name_hit:
-            raise TypeError(
-                f"The following tags have multiple configs: {has_multi_configs} but the config `{name}`  was found only in: {config_name_hit}."
-            )
-        if config_name_hit == 0:
-            logger.warning(
-                "No matching config names found in the metadata, using the common values to create metadata."
-            )
 
-        return DatasetMetadata(**result_dict)
+# In general the allowed YAML keys are the fields of the DatasetMetadata dataclass.
+# However it is not the case certain fields like train_eval_index,
+# for which the YAML key must use dashes and not underscores.
+# Fields that corresponds to YAML keys with dashes are defined in DatasetMetadata._FIELDS_WITH_DASHES
+DatasetMetadata._ALLOWED_YAML_KEYS = {
+    field.name.replace("_", "-") if field.name in DatasetMetadata._FIELDS_WITH_DASHES else field.name
+    for field in fields(DatasetMetadata)
+}
 
 
 if __name__ == "__main__":
